"""Formatting of pyparams style args to execute.
"""
import datetime

# Sibling Imports
from pgtypes import *
from errors import *

# Types that are already suitable for str() representation
donetypes  = [STRING,BINARY,NUMBER,ROWID,DATETIME,BOOL,int,float]

def typeCastParam(p):
	"""Cast p to the appropriate type if possible.
	"""
	if p is None:
		return NULL()

	tp = type(p)
	if tp in donetypes:
		return p
	
	if tp is str:
		return STRING(p)
 	if tp is unicode:
 		return STRING(p.encode('utf-8'))
	if tp is datetime.datetime:
		return DATETIME(p.year, p.month, p.day, 
		p.hour, p.minute, p.second, p.microsecond)
	if tp is datetime.date:
		return Date(p.year, p.month, p.day)
	if tp is datetime.time:
		return Time(p.hour, p.minute, p.second, p.microsecond)
	if tp is bool:
		return Bool(p)

	raise ProgrammingError, ("Query parameter: cannot convert %s to a PostgreSQL data type" % tp)

def format(s,params):
	"""Make params appear in s correctly.

	This includes pgsql specific date/time formats, safe strings, 
	binary sequences, etc
	"""

	s = s.strip()
	if s[-1] == ";":
		s = s[:-1]

	if params is None:
		return s

	if isinstance(params,tuple):
		tmp = tuple([typeCastParam(p) for p in params])
		params = tmp
	elif isinstance(params,dict):
		for k,v in params.items():
			params[k] = typeCastParam(v)
	else:
		params = typeCastParam(params)
	
	return s % params
